package com.simuwang.manage.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import com.alibaba.excel.EasyExcel;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.common.util.EmailUtil;
import com.simuwang.base.config.CompetitionConfig;
import com.simuwang.base.pojo.dto.competition.CompetitionBaseResultDTO;
import com.simuwang.base.pojo.dto.FundNavDataDTO;
import com.simuwang.base.pojo.dto.FundNavDeletionDTO;
import com.simuwang.base.pojo.dto.MailboxInfoDTO;
import com.simuwang.manage.service.EmailSystemConfigService;
import com.simuwang.manage.service.competition.FundNavService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.Date;
import java.util.List;

@Component
public class CompetitionTask {

    private static final Logger log = LoggerFactory.getLogger(CompetitionTask.class);

    private final FundNavService fundNavService;
    private final CompetitionConfig competitionConfig;

    private final EmailSystemConfigService emailSystemConfigService;

    public CompetitionTask(FundNavService fundNavService, CompetitionConfig competitionConfig, EmailSystemConfigService emailSystemConfigService) {
        this.fundNavService = fundNavService;
        this.competitionConfig = competitionConfig;
        this.emailSystemConfigService = emailSystemConfigService;
    }

    /**
     * 输出基金净值采集结果明细任务
     * 每周五 18:00 执行
     */
    //@Scheduled(cron = "0 0 18 ? * 5")
    public void fundNavDataTask() {
        List<FundNavDataDTO> navDataDTOList = fundNavService.getFungNavData();
        if (CollUtil.isEmpty(navDataDTOList)) {
            return;
        }
        String filePath = competitionConfig.getDirectory() + "基金净值采集结果明细_" + DateUtil.format(new Date(), DateConst.YYYYMMDD) + ".xlsx";
        EasyExcel.write(filePath, FundNavDataDTO.class)
                .sheet("基金净值采集结果明细")
                .doWrite(navDataDTOList);
        log.info("基金净值采集结果明细表格已输出 -> 文件:{}", filePath);

        Integer method = competitionConfig.getMethod();
        // 通过邮件发送"基金净值采集结果明细表格"
        if (method == 1) {
            MailboxInfoDTO mailboxInfoDTO = emailSystemConfigService.getFromEmailInfo();
            String receivingMailbox = emailSystemConfigService.getRecipientEmail();
            try {
                EmailUtil.senEmail(mailboxInfoDTO, receivingMailbox, new File(filePath), "基金净值采集结果明细", "", "基金净值采集结果明细");
                log.info("基金净值采集结果明细表 -> 邮件已发送");
            } catch (Exception e) {
                log.error("邮件发送基金净值采集结果明细表异常 -> 堆栈信息:{}", ExceptionUtil.stacktraceToString(e));
            }
        }
    }

    /**
     * 输出基金净值缺失明细任务
     * 每周五 18:00 执行
     */
   // @Scheduled(cron = "0 0 18 ? * 5")
    public void navDeletionTask() {
        List<FundNavDeletionDTO> navDeletionDTOList = fundNavService.getFundNavDeletion();
        String filePath = competitionConfig.getDirectory() + "基金净值缺失明细_" + DateUtil.format(new Date(), DateConst.YYYYMMDD) + ".xlsx";
        EasyExcel.write(filePath, FundNavDeletionDTO.class)
                .sheet("基金净值缺失明细")
                .doWrite(navDeletionDTOList);
        log.info("基金净值缺失明细表格已输出 -> 文件:{}", filePath);

        Integer method = competitionConfig.getMethod();
        // 通过邮件发送"基金净值采集结果明细表格"
        if (method == 1) {
            MailboxInfoDTO mailboxInfoDTO = emailSystemConfigService.getFromEmailInfo();
            String recipientEmail = emailSystemConfigService.getRecipientEmail();
            try {
                EmailUtil.senEmail(mailboxInfoDTO, recipientEmail, new File(filePath), "基金净值缺失明细", "", "基金净值缺失明细");
                log.info("基金净值缺失明细表 -> 邮件已发送");
            } catch (Exception e) {
                log.error("邮件发送基金净值缺失明细表异常 -> 堆栈信息:{}", ExceptionUtil.stacktraceToString(e));
            }
        }
    }

   // @Scheduled(cron = "0/10 * * * * ?")
    public void competitionResultTask() {
        Integer competitionId = competitionConfig.getId();
        String competitionName = competitionConfig.getName();
        List<? extends CompetitionBaseResultDTO> competitionResultDTOList = fundNavService.getCompetitionResult(competitionId);
        if (CollUtil.isEmpty(competitionResultDTOList)) {
            return;
        }
        Class<? extends CompetitionBaseResultDTO> aClass = competitionResultDTOList.get(0).getClass();
        String filePath = competitionConfig.getDirectory() + competitionName+ "_基金净值衍生指标及排名结果明细_" + DateUtil.format(new Date(), DateConst.YYYYMMDD) + ".xlsx";
        EasyExcel.write(filePath, aClass)
                .sheet("基金净值衍生指标及排名结果明细")
                .doWrite(competitionResultDTOList);
        log.info("基金净值衍生指标及排名结果明细表已输出 -> 文件:{}", filePath);

        Integer method = competitionConfig.getMethod();
        // 通过邮件发送"基金净值采集结果明细表格"
        if (method == 2) {
            MailboxInfoDTO mailboxInfoDTO = emailSystemConfigService.getFromEmailInfo();
            String recipientEmail = emailSystemConfigService.getRecipientEmail();
            try {
                EmailUtil.senEmail(mailboxInfoDTO, recipientEmail, new File(filePath), "基金净值衍生指标及排名结果明细", "", "基金净值衍生指标及排名结果明细");
                log.info("基金净值衍生指标及排名结果明细表 -> 邮件已发送");
            } catch (Exception e) {
                log.error("邮件发送基金净值衍生指标及排名结果明细表异常 -> 堆栈信息:{}", ExceptionUtil.stacktraceToString(e));
            }
        }
    }

}
